@@ -22,14 +22,15 @@ import datetime
 from copy import deepcopy
 
 from pymatgen.core.structure import Structure
-
 from pymatgen.io.cifio import CifParser
 from pymatgen.io.vaspio.vasp_input import Poscar
 from pymatgen.serializers.json_coders import MSONable, PMGJSONDecoder
+from pymatgen.matproj.snl import StructureNL
+
+from warnings import warn
 
 dec = PMGJSONDecoder()
 
-
 class TransformedStructure(MSONable):
     """
     Container object for new structures that include history of
@@ -38,49 +39,18 @@ class TransformedStructure(MSONable):
     Each transformed structure is made up of a sequence of structures with
     associated transformation history.
     """
-
-    def __init__(self, structure, transformations, history=None,
+    
+    def __init__(self, structure, transformations=None, history=None,
                  other_parameters=None):
-        """
-        Standard constructor for a TransformedStructure.
-
-        Args:
-            structure:
-                input structure
-            transformations:
-                Sequence of transformations to be applied to the input
-                structure.
-            history:
-                optional history for the input structure, which provides a way
-                to track structures having undergone multiple series of
-                transformations.
-            other_parameters:
-                optional parameters to store along with the
-                TransformedStructure. This can include tags (a list) or author
-                which will be parsed.
-        """
-        history = [] if history is None else history
-        self._source = {}
-        self._structures = []
-        self._changes = []
-        self._change_parameters = []
-        self._redo_trans = []
-        self._other_parameters = {} if other_parameters is None \
-            else deepcopy(other_parameters)
-        if len(history) > 0:
-            self._source = history[0]
-            for i in xrange(1, len(history)):
-                struct = Structure.from_dict(history[i]["input_structure"])
-                trans = dec.process_decoded(history[i])
-                param = history[i].get("output_parameters", {})
-                self._structures.append(struct)
-                self._changes.append(trans)
-                self._change_parameters.append(param)
-
-        self._structures.append(structure)
+        self.final_structure = structure
+        self.history = history or []
+        self.other_parameters = other_parameters or {}
+        self._undone = []
+        
+        transformations = transformations or []
         for t in transformations:
             self.append_transformation(t)
-
+            
     def undo_last_change(self):
         """
         Undo the last change in the TransformedStructure.
@@ -88,11 +58,16 @@ class TransformedStructure(MSONable):
         Raises:
             IndexError if already at the oldest change.
         """
-        if len(self._changes) == 0:
+        if len(self.history) == 0:
             raise IndexError("Can't undo. Already at oldest change.")
-        self._structures.pop()
-        self._change_parameters.pop()
-        self._redo_trans.append(self._changes.pop())
+        if 'input_structure' not in self.history[-1]:
+            raise IndexError("Can't undo. Latest history has no input_structure")
+        h = self.history.pop()
+        self._undone.append((h, self.final_structure))
+        s = h["input_structure"]
+        if isinstance(s, dict):
+            s = Structure.from_dict(s)
+        self.final_structure = s
 
     def redo_next_change(self):
         """
@@ -101,24 +76,19 @@ class TransformedStructure(MSONable):
         Raises:
             IndexError if already at the latest change.
         """
-        if len(self._redo_trans) == 0:
-            raise IndexError("Can't undo. Already at latest change.")
-        t = self._redo_trans.pop()
-        if hasattr(t, 'apply_transformation'):
-            self.append_transformation(t, clear_redo=False)
-        else:
-            self.append_filter(t)
-
-    def __getitem__(self, index):
-        return self._structures[index], self._changes[0:index]
-
+        if len(self._undone) == 0:
+            raise IndexError("Can't redo. Already at latest change.")
+        h, s = self._undone.pop()
+        self.history.append(h)
+        self.final_structure = s
+        
     def __getattr__(self, name):
-        s = object.__getattribute__(self, '_structures')[-1]
+        s = object.__getattribute__(self, 'final_structure')
         return getattr(s, name)
-
+    
     def __len__(self):
-        return len(self._structures)
-
+        return len(self.history)
+    
     def append_transformation(self, transformation, return_alternatives=False,
                               clear_redo=True):
         """
@@ -140,48 +110,51 @@ class TransformedStructure(MSONable):
                 redos.
         """
         if clear_redo:
-            self._redo_trans = []
-
+            self._undone = []
+            
         if return_alternatives and transformation.is_one_to_many:
-            starting_struct = self._structures[-1]
             ranked_list = transformation.apply_transformation(
-                starting_struct, return_ranked_list=return_alternatives)
-            #generate the alternative structures
+                self.final_structure, return_ranked_list=return_alternatives)
+            
+            input_structure = self.final_structure.to_dict
             alts = []
             for x in ranked_list[1:]:
-                struct = x.pop("structure")
-                other_paras = self._other_parameters.copy()
-                hist = self.history
+                s = x.pop("structure")
                 actual_transformation = x.pop("transformation", transformation)
-                tdict = actual_transformation.to_dict
-                tdict["input_structure"] = starting_struct.to_dict
-                tdict["output_parameters"] = x
-                hist.append(tdict)
-                alts.append(TransformedStructure(struct, [], history=hist,
-                                                 other_parameters=other_paras))
-            #use the first item in the ranked_list and apply it to this
-            #transformed_structure
+                hdict = actual_transformation.to_dict
+                hdict["input_structure"] = input_structure
+                hdict["output_parameters"] = x
+                self.final_structure = s
+                d = self.to_dict
+                d['history'].append(hdict)
+                d['final_structure'] = s.to_dict
+                alts.append(TransformedStructure.from_dict(d))
+                
             x = ranked_list[0]
-            struct = x.pop("structure")
+            s = x.pop("structure")
             actual_transformation = x.pop("transformation", transformation)
-            self._structures.append(struct)
-            self._changes.append(actual_transformation)
-            self._change_parameters.append(x)
+            hdict = actual_transformation.to_dict
+            hdict["input_structure"] = self.final_structure.to_dict
+            hdict["output_parameters"] = x
+            self.history.append(hdict)
+            self.final_structure = s
             return alts
         else:
-            new_s = transformation.apply_transformation(self._structures[-1])
-            self._structures.append(new_s)
-            self._change_parameters.append({})
-            self._changes.append(transformation)
-
+            s = transformation.apply_transformation(self.final_structure)
+            hdict = transformation.to_dict
+            hdict["input_structure"] = self.final_structure.to_dict
+            hdict["output_parameters"] = {}
+            self.history.append(hdict)
+            self.final_structure = s
+            
     def append_filter(self, structure_filter):
         """
         Adds a transformation parameter to the last transformation.
         """
-        self._structures.append(self._structures[-1])
-        self._change_parameters.append({})
-        self._changes.append(structure_filter)
-
+        hdict = structure_filter.to_dict
+        hdict["input_structure"] = self.final_structure.to_dict
+        self.history.append(hdict)
+        
     def extend_transformations(self, transformations):
         """
         Extends a sequence of transformations to the TransformedStructure.
@@ -192,7 +165,7 @@ class TransformedStructure(MSONable):
         """
         for t in transformations:
             self.append_transformation(t)
-
+            
     def get_vasp_input(self, vasp_input_set, generate_potcar=True):
         """
         Returns VASP input as a dict of vaspio objects.
@@ -206,7 +179,7 @@ class TransformedStructure(MSONable):
                 POTCAR, which contains the POTCAR labels but not the actual
                 POTCAR. Defaults to True.
         """
-        d = vasp_input_set.get_all_vasp_input(self._structures[-1],
+        d = vasp_input_set.get_all_vasp_input(self.final_structure,
                                               generate_potcar)
         d["transformations.json"] = json.dumps(self.to_dict)
         return d
@@ -225,33 +198,27 @@ class TransformedStructure(MSONable):
             create_directory:
                 Create the directory if not present. Defaults to True.
         """
-        vasp_input_set.write_input(self._structures[-1], output_dir,
+        vasp_input_set.write_input(self.final_structure, output_dir,
                                    make_dir_if_not_present=create_directory)
         with open(os.path.join(output_dir, "transformations.json"), "w") as fp:
             json.dump(self.to_dict, fp)
 
     def __str__(self):
         output = ["Current structure", "------------",
-                  str(self._structures[-1]),
-                  "\nSource", "------------",
-                  str(self._source),
-                  "\nTransformation history",
+                  str(self.final_structure),
+                  "\nHistory",
                   "------------"]
-        for i, t in enumerate(self._changes):
-            output.append("{} {}".format(t.to_dict,
-                                         self._change_parameters[i]))
+        for h in self.history:
+            h.pop('input_structure', None)
+            output.append(str(h))
         output.append("\nOther parameters")
         output.append("------------")
-        output.append(str(self._other_parameters))
+        output.append(str(self.other_parameters))
         return "\n".join(output)
-
+    
     def set_parameter(self, key, value):
-        self._other_parameters[key] = value
-
-    @property
-    def other_parameters(self):
-        return self._other_parameters
-
+        self.other_parameters[key] = value
+    
     @property
     def was_modified(self):
         """
@@ -260,62 +227,18 @@ class TransformedStructure(MSONable):
         is in the case of performing a substitution transformation on the
         structure when the specie to replace isn't in the structure.
         """
-        return not self._structures[-1] == self._structures[-2]
-
+        return not self.final_structure == self.structures[-2]
+    
     @property
     def structures(self):
         """
         Returns a copy of all structures in the TransformedStructure. A
         structure is stored after every single transformation.
         """
-        return [s for s in self._structures]
-
-    @property
-    def transformations(self):
-        """
-        Returns a copy of all transformations in the TransformedStructure.
-        """
-        return [t for t in self._changes]
-
-    @property
-    def final_structure(self):
-        """
-        Returns the final structure in the TransformedStructure.
-        """
-        return self._structures[-1]
-
-    @classmethod
-    def from_dict(cls, d):
-        """
-        Creates a TransformedStructure from a dict.
-        """
-        s = Structure.from_dict(d)
-        return cls(s, [], d["history"], d.get("other_parameters", None))
-
-    @property
-    def history(self):
-        history = [self._source]
-        for i, t in enumerate(self._changes):
-            tdict = t.to_dict
-            tdict["input_structure"] = self._structures[i].to_dict
-            tdict["output_parameters"] = self._change_parameters[i]
-            history.append(tdict)
-        return history
-
-    @property
-    def to_dict(self):
-        """
-        Returns a dict representation of the TransformedStructure.
-        """
-        d = self._structures[-1].to_dict
-        d["@module"] = self.__class__.__module__
-        d["@class"] = self.__class__.__name__
-        d["history"] = self.history
-        d["version"] = __version__
-        d["last_modified"] = str(datetime.datetime.utcnow())
-        d["other_parameters"] = self._other_parameters
-        return d
-
+        hstructs = [Structure.from_dict(s['input_structure']) for s in self.history \
+                    if 'input_structure' in s]
+        return hstructs + [self.final_structure]
+            
     @staticmethod
     def from_cif_string(cif_string, transformations=None, primitive=True,
                         occupancy_tolerance=1.):
@@ -353,7 +276,7 @@ class TransformedStructure(MSONable):
                        "datetime": str(datetime.datetime.now()),
                        "original_file": raw_string,
                        "cif_data": cif_dict[cif_keys[0]]}
-        return TransformedStructure(s, transformations, [source_info])
+        return TransformedStructure(s, transformations, history=[source_info])
 
     @staticmethod
     def from_poscar_string(poscar_string, transformations=None):
@@ -376,4 +299,64 @@ class TransformedStructure(MSONable):
         source_info = {"source": "POSCAR",
                        "datetime": str(datetime.datetime.now()),
                        "original_file": raw_string}
-        return TransformedStructure(s, transformations, [source_info])
+        return TransformedStructure(s, transformations, history=[source_info])
+    
+    @property
+    def to_dict(self):
+        """
+        Returns a dict representation of the TransformedStructure.
+        """
+        d = self.final_structure.to_dict
+        d["@module"] = self.__class__.__module__
+        d["@class"] = self.__class__.__name__
+        d["history"] = deepcopy(self.history)
+        d["version"] = __version__
+        d["last_modified"] = str(datetime.datetime.utcnow())
+        d["other_parameters"] = deepcopy(self.other_parameters)
+        return d
+    
+    @classmethod
+    def from_dict(cls, d):
+        """
+        Creates a TransformedStructure from a dict.
+        """
+        s = Structure.from_dict(d)
+        return cls(s, history=d["history"], 
+                   other_parameters=d.get("other_parameters", None))
+        
+    def to_snl(self, authors, projects=None, references='', remarks=None,
+               data=None, created_at=None):
+        if self.other_parameters:
+            warn('Data in TransformedStructure.other_parameters discarded '
+                 'during type conversion to SNL')
+        hist = []
+        for h in self.history:
+            snl_metadata = h.pop('_snl', {})
+            hist.append({'name' : snl_metadata.pop('name', 'pymatgen'),
+                         'url' : snl_metadata.pop('url', 
+                                    'http://pypi.python.org/pypi/pymatgen'),
+                         'description' : h})
+        return StructureNL(self.final_structure, authors, projects, references,
+                           remarks, data, hist, created_at)
+        
+    @classmethod
+    def from_snl(cls, snl):
+        """
+        Args:
+            snl:
+                Starting snl
+            copy_metadata:
+                update the authors, projects, references, and remarks
+                in the last history node with the metadata at the
+                root of the SNL object
+            copy_data:
+                copy the contents of snl.data into the last history
+                node
+        """
+        hist = []
+        for h in snl.history:
+            d = h.description
+            d['_snl'] = {'url' : h.url, 'name' : h.name}
+            hist.append(d)
+        return cls(snl.structure, history=hist)
+
