@@ -1,6 +1,6 @@
 """
-This module provides objects describing the basic parameters of the pseudopotentials used in Abinit,
-a parser to instantiate pseudopotential objects from file and a simple database to access the official pseudopotential tables.
+This module provides objects describing the basic parameters of the 
+pseudopotentials used in Abinit, and a parser to instantiate pseudopotential objects..
 """
 from __future__ import division, print_function
 
@@ -14,33 +14,24 @@ import cPickle as pickle
 import cStringIO as StringIO
 import numpy as np
 
-from warnings import warn
-from pprint import pprint
-
 from pymatgen.core.periodic_table import PeriodicTable
 from pymatgen.core.physical_constants import Ha_eV, Ha2meV
 from pymatgen.util.num_utils import iterator_from_slice
-#from pymatgen.serializers.json_coders import MSONable
 
 __all__ = [
-'Pseudo',
-'PseudoDatabase',
+    "Pseudo",
+    "PseudoTable",
 ]
 
 __author__ = "Matteo Giantomassi"
-__copyright__ = "Copyright 2013, The Materials Project"
 __version__ = "0.1"
 __maintainer__ = "Matteo Giantomassi"
-__email__ = "gmatteo at gmail.com"
-__status__ = "Development"
-__date__ = "$Feb 21, 2013M$"
 
 ##########################################################################################
 # Tools and helper functions.
 
 class FrozenDict(dict):
-    "A dictionary that does not permit to redefine its keys"
-
+    """A dictionary that does not permit to redefine its keys."""
     def __init__(self, *args, **kwargs):
         self.update(*args, **kwargs)
 
@@ -53,16 +44,6 @@ class FrozenDict(dict):
         for (k, v) in dict(*args, **kwargs).iteritems():
             self[k] = v
 
-def nested_dict_items(nested):
-    "Iterate over the items of a nested Mapping (e.g. a dictionary)."
-
-    for (key, value) in nested.items():
-        if isinstance(value, collections.Mapping):
-            for (inner_key, inner_value) in nested_dict_items(value):
-                yield inner_key, inner_value
-        else:
-            yield key, value
-
 def _read_nlines(filename, nlines):
     """
     Read at most nlines lines from file filename.
@@ -79,17 +60,6 @@ def _read_nlines(filename, nlines):
             lines.append(line)
         return lines
 
-def read_dojo_report(filename):
-    with open(filename, "r") as fh:
-         lines = fh.readlines()
-         try:
-             start = lines.index("<DOJO_REPORT>\n")
-         except ValueError:
-             return {}
-         stop = lines.index("</DOJO_REPORT>\n")
-
-         return json.loads("".join(l for l in lines[start+1:stop]))
-
 _l2str = {
     0: "s",
     1: "p",
@@ -101,35 +71,45 @@ _l2str = {
 }
 
 def l2str(l):
-    "Convert the angular momentum l (int) to string."
+    """Convert the angular momentum l (int) to string."""
     try:
         return _l2str[l]
     except KeyError:
         return "Unknown: received l = %s" % l
 
-# TODO
-# Should become an API common for the different codes that requires pseudos
-def get_abinit_psp_dir(code="ABINIT"):
-    import ConfigParser
-    import pymatgen
+def read_dojo_report(filename):
+    """Helper function to read the DOJO_REPORT from file."""
+    with open(filename, "r") as fh:
+        lines = fh.readlines()
+        try:
+            start = lines.index("<DOJO_REPORT>\n")
+        except ValueError:
+            return {}
 
-    if code + "_PSP_DIR" in os.environ:
-        return os.environ[code + "_PSP_DIR"]
+        stop = lines.index("</DOJO_REPORT>\n")
+        return json.loads("".join(lines[start+1:stop]))
 
-    elif os.path.exists(os.path.join(os.path.dirname(pymatgen.__file__), "pymatgen.cfg")):
-        module_dir = os.path.dirname(pymatgen.__file__)
-        config = ConfigParser.SafeConfigParser()
-        config.readfp(open(os.path.join(module_dir, "pymatgen.cfg")))
-        return config.get(code, "pspdir")
-
-    return None
+#class DojoReport(dict):
+#    _LATEST_VERSION = 1.0
+#    _START_LINE = "<DOJO_REPORT>\n"
+#    _END_LINE = "</DOJO_REPORT>\n"
+#
+#    @classmethod
+#    def from_file(cls, path):
+#        new = read_dojo_report(path)
+#        new.__class__ = cls
+#        return new
+#
+#    #def to_file(self, path):
 
 ##########################################################################################
-_periodic_table = PeriodicTable()
+
+_PTABLE = PeriodicTable()
 
 class Pseudo(object):
     """
-    Abstract base class defining the methods that must be implemented by the concrete pseudopotential classes.
+    Abstract base class defining the methods that must be 
+    implemented by the concrete pseudopotential classes.
     """
     __metaclass__ = abc.ABCMeta
 
@@ -139,7 +119,7 @@ class Pseudo(object):
         Convert obj into a pseudo. Accepts:
 
             * Pseudo object.
-            * string defining a path.
+            * string defining a valid path.
         """
         if isinstance(obj, cls):
             return obj
@@ -160,7 +140,7 @@ class Pseudo(object):
             self.__class__.__name__, id(self), self.name)
 
     def __str__(self):
-        "String representation"
+        """String representation."""
         lines = []
         app = lines.append
         app("<%s: %s>" % (self.__class__.__name__, self.name))
@@ -170,108 +150,118 @@ class Pseudo(object):
         app("  maximum angular momentum: %s" % l2str(self.l_max))
         app("  angular momentum for local part: %s" % l2str(self.l_local))
         app("  radius for non-linear core correction: %s" % self.nlcc_radius)
+        app("")
+
+        hint_normal = self.hint_for_accuracy()
+        if hint_normal is not None:
+            app("  hint for normal accuracy: %s" % str(hint_normal))
+
         return "\n".join(lines)
 
     @abc.abstractproperty
     def summary(self):
-        "String summarizing the most important properties"
+        """String summarizing the most important properties."""
 
     @abc.abstractproperty
     def filepath(self):
-        "Absolute path of the pseudopotential file"
+        """Absolute path of the pseudopotential file."""
 
     @property
     def name(self):
-        "File basename"
+        """File basename."""
         return os.path.basename(self.filepath)
 
     @abc.abstractproperty
     def Z(self):
-        "The atomic number of the atom."
+        """The atomic number of the atom."""
 
     @abc.abstractproperty
     def Z_val(self):
-        "Valence charge"
+        """Valence charge"""
 
     @property
     def element(self):
-        "Pymatgen Element"
-        return _periodic_table[self.Z]
+        """Pymatgen `Element`."""
+        return _PTABLE[self.Z]
 
     @property
     def symbol(self):
-        "Element symbol."
+        """Element symbol."""
         return self.element.symbol
 
     @abc.abstractproperty
     def l_max(self):
-        "Maximum angular momentum"
+        """Maximum angular momentum."""
 
     @abc.abstractproperty
     def l_local(self):
-        "Angular momentum used for the local part"
+        """Angular momentum used for the local part."""
 
     #@abc.abstractproperty
-    #def xc_family(self):
-    #    "XC family e.g LDA, GGA, MGGA."
+    #def xc_type(self):
+    #    """XC family e.g LDA, GGA, MGGA."""
 
     #@abc.abstractproperty
     #def xc_flavor(self):
-    #    "XC flavor e.g PW, PW91, PBE."
+    #    """XC flavor e.g PW, PW91, PBE."""
 
-    @property
-    def xc_type(self):
-        "XC identifier e.g LDA-PW, GGA-PBE, GGA-revPBE"
-        return "-".join([self.xc_family, self.xc_flavor])
+    #@property
+    #def xc_functional(self):
+    #    """XC identifier e.g LDA-PW91, GGA-PBE, GGA-revPBE."""
+    #    return "-".join([self.xc_type, self.xc_flavor])
 
     @property
     def isnc(self):
-        "True if norm-conserving pseudopotential"
+        """True if norm-conserving pseudopotential."""
         return isinstance(self, NcPseudo)
 
     @property
     def ispaw(self):
-        "True if PAW pseudopotential"
+        """True if PAW pseudopotential."""
         return isinstance(self, PawPseudo)
 
     #@abc.abstractproperty
     #def has_soc(self):
-    #    "True if pseudo contains spin-orbit coupling."
+    #    """True if pseudo contains spin-orbit coupling."""
 
     #@abc.abstractmethod
     #def num_of_projectors(self, l='s'):
-    #    "Number of projectors for the angular channel l"
+    #    """Number of projectors for the angular channel l"""
 
     #@abc.abstractmethod
     #def generation_mode
-    #    "scalar scalar-relativistic, relativistic"
+    #    """scalar scalar-relativistic, relativistic."""
 
     @property
     def has_dojo_report(self):
-        "True if self contains the DOJO_REPORT section."
-        return self.dojo_report
+        """True if self contains the DOJO_REPORT section."""
+        return bool(self.dojo_report)
 
-    @property
-    def dojo_level(self):
-        "The maximum level of the DOJO trials passes by the pseudo."
-        if not self.dojo_report:
+    def delta_factor(self, accuracy="normal"):
+        """
+        Returns the deltafactor [meV/natom] computed with the given accuracy.
+        None if self does not have info on the deltafactor.
+        """
+        if not self.has_dojo_report:
+            return None
+        try:
+            return self.dojo_report["delta_factor"][accuracy]["dfact"]
+        except KeyError:
             return None
-        else:
-            from pymatgen.io.abinitio.pseudo_dojo import dojo_key2level
-            levels = [dojo_key2level(key) for key in self.dojo_report]
-            return max(levels)
-
-    #def dojo_rank(self):
 
     def read_dojo_report(self):
+        """Read the DOJO_REPORT section, returns {} if section is not present.""" 
         return read_dojo_report(self.path)
 
     def write_dojo_report(self, report):
+        """Write a new DOJO_REPORT section to the pseudopotential file."""
+        path = self.path
+
         # Create JSON string from report.
         jstring = json.dumps(report, indent=4, sort_keys=True) + "\n"
 
         # Read lines from file and insert jstring between the tags.
-        with open(self.path, "r") as fh:
+        with open(path, "r") as fh:
             lines = fh.readlines()
             try:
                 start = lines.index("<DOJO_REPORT>\n")
@@ -287,12 +277,14 @@ class Pseudo(object):
                del lines[start+1:stop]
 
         #  Write new file.
-        with open(self.path, "w") as fh:
+        with open(path, "w") as fh:
             fh.writelines(lines)
 
     def remove_dojo_report(self):
+        """Remove the DOJO_REPORT section from the pseudopotential file."""
         # Read lines from file and insert jstring between the tags.
-        with open(self.path, "r") as fh:
+        path = self.path
+        with open(path, "r") as fh:
             lines = fh.readlines()
             try:
                 start = lines.index("<DOJO_REPORT>\n")
@@ -307,16 +299,15 @@ class Pseudo(object):
                return
 
             del lines[start+1:stop]
-            #pprint(lines)
 
         # Write new file.
-        with open(self.path, "w") as fh:
+        with open(path, "w") as fh:
             fh.writelines(lines)
 
-    def hint_for_accuracy(self, accuracy):
+    def hint_for_accuracy(self, accuracy="normal"):
         """
-        Returns an hint object with parameters such as ecut and aug_ratio for
-        given accuracy. Returns None if no hint is available.
+        Returns an hint object with parameters such as ecut [Ha] and 
+        aug_ratio for given accuracy. Returns None if no hint is available.
 
         Args:
             accuracy: ["low", "normal", "high"]
@@ -328,26 +319,22 @@ class Pseudo(object):
 
     @property
     def has_hints(self):
-        "True if self provides hints on the cutoff energy"
+        """True if self provides hints on the cutoff energy."""
         for acc in ["low", "normal", "high"]:
             if self.hint_for_accuracy(acc) is None:
                 return False
         return True
 
-    def checksum(self):
-        """
-        Return the checksum of the pseudopotential file.
-
-        The checksum is given by the tuple (basename, line_num, hexmd5)
-        where basename if the file name, hexmd5 is the (hex) MD5 hash,
-        and line_num is the number of lines in the file.
-        """
-        import hashlib
-        hasher = hashlib.md5()
-        with open(self.filepath, "r") as fh:
-            hasher.update(fh.read())
-
-        return self.name, len(text.splitlines()), hasher.hexdigest()
+    #@property
+    #def md5(self):
+    #    """
+    #    Return the checksum of the pseudopotential file.
+    #    """
+    #    import hashlib
+    #    hasher = hashlib.md5()
+    #    with open(self.filepath, "r") as fh:
+    #        hasher.update(fh.read())
+    #        return hasher.hexdigest()
 
 ##########################################################################################
 
@@ -367,7 +354,7 @@ class NcPseudo(object):
 
     @property
     def has_nlcc(self):
-        "True if the pseudo is generated with non-linear core correction."
+        """True if the pseudo is generated with non-linear core correction."""
         return self.nlcc_radius > 0.0
 
 ##########################################################################################
@@ -381,7 +368,7 @@ class PawPseudo(object):
 
     @abc.abstractproperty
     def paw_radius(self):
-        "Radius of the PAW sphere in a.u."
+        """Radius of the PAW sphere in a.u."""
 
 ##########################################################################################
 
@@ -390,8 +377,14 @@ class AbinitPseudo(Pseudo):
     An AbinitPseudo is a pseudopotential whose file contains an abinit header.
     """
     def __init__(self, path, header):
-
-        self.path     = path
+        """
+        Args:
+            path:
+                Filename.
+            header:
+                `AbinitHeader` instance.
+        """
+        self.path = path
         self._summary = header.summary
         self.dojo_report = header.dojo_report
         #self.pspcod  = header.pspcod
@@ -404,6 +397,7 @@ class AbinitPseudo(Pseudo):
 
     @property
     def summary(self):
+        """Summary line reported in the ABINIT header."""
         return self._summary.strip()
 
     @property
@@ -416,7 +410,6 @@ class AbinitPseudo(Pseudo):
 
     @property
     def Z_val(self):
-        "Number of valence electrons."
         return self._zion
 
     @property
@@ -433,7 +426,6 @@ class NcAbinitPseudo(NcPseudo, AbinitPseudo):
     """
     Norm-conserving pseudopotential in the Abinit format.
     """
-
     @property
     def summary(self):
         return self._summary.strip()
@@ -448,7 +440,7 @@ class NcAbinitPseudo(NcPseudo, AbinitPseudo):
 
     @property
     def Z_val(self):
-        "Number of valence electrons."
+        """Number of valence electrons."""
         return self._zion
 
     @property
@@ -466,7 +458,7 @@ class NcAbinitPseudo(NcPseudo, AbinitPseudo):
 ##########################################################################################
 
 class PawAbinitPseudo(PawPseudo, AbinitPseudo):
-    "Paw pseudopotential in the Abinit format."
+    """Paw pseudopotential in the Abinit format."""
 
     @property
     def paw_radius(self):
@@ -482,8 +474,6 @@ class Hint(collections.namedtuple("Hint", "ecut aug_ratio")):
     """
     @property
     def to_dict(self):
-        #d["@module"] = self.__class__.__module__
-        #d["@class"] = self.__class__.__name__
         return {f: getattr(self, f) for f in self._fields}
 
     @classmethod
@@ -504,7 +494,6 @@ def _dict_from_lines(lines, key_nums, sep=None):
 
     Return dict{key1 : value1, key2 : value2, ...}
     """
-
     if isinstance(lines, basestring):
         lines = [lines]
 
@@ -542,8 +531,7 @@ def _dict_from_lines(lines, key_nums, sep=None):
 ##########################################################################################
 
 class AbinitHeader(dict):
-    "Dictionary whose keys can be also accessed as attributes."
-
+    """Dictionary whose keys can be also accessed as attributes."""
     def __getattr__(self, name):
         try:
             # Default behaviour
@@ -558,6 +546,12 @@ class AbinitHeader(dict):
 ##########################################################################################
 
 def _int_from_str(string):
+    """
+    Convert string into integer
+
+    Raise:
+        TypeError if string is not a valid integer
+    """
     float_num = float(string)
     int_num = int(float_num)
     if float_num == int_num:
@@ -571,7 +565,7 @@ class NcAbinitHeader(AbinitHeader):
     """
     _attr_desc = collections.namedtuple("att", "default astype")
 
-    _vars = {
+    _VARS = {
         # Mandatory
         "zatom"        : _attr_desc(None, _int_from_str),
         "zion"         : _attr_desc(None, float),
@@ -590,12 +584,11 @@ class NcAbinitHeader(AbinitHeader):
     del _attr_desc
 
     def __init__(self, summary, **kwargs):
-
         super(NcAbinitHeader, self).__init__()
 
         self.summary = summary.strip()
 
-        for (key, desc) in NcAbinitHeader._vars.items():
+        for (key, desc) in NcAbinitHeader._VARS.items():
             default, astype = desc.default, desc.astype
 
             value = kwargs.pop(key, None)
@@ -616,18 +609,16 @@ class NcAbinitHeader(AbinitHeader):
         self["dojo_report"] = kwargs.pop("dojo_report", {})
 
         if kwargs:
-            msg = "kwargs should be empty but got %s" % str(kwargs)
-            raise RuntimeError(msg)
+            raise RuntimeError("kwargs should be empty but got %s" % str(kwargs))
 
     @staticmethod
     def fhi_header(filename, ppdesc):
-        "Parse the FHI abinit header."
+        """Parse the FHI abinit header."""
         # Example:
         # Troullier-Martins psp for element  Sc        Thu Oct 27 17:33:22 EDT 1994
         #  21.00000   3.00000    940714                zatom, zion, pspdat
         #    1    1    2    0      2001    .00000      pspcod,pspxc,lmax,lloc,mmax,r2well
         # 1.80626423934776     .22824404341771    1.17378968127746   rchrg,fchrg,qchrg
-
         lines = _read_nlines(filename, -1)
 
         header = _dict_from_lines(lines[:4], [0, 3, 6, 3])
@@ -639,12 +630,11 @@ class NcAbinitHeader(AbinitHeader):
 
     @staticmethod
     def hgh_header(filename, ppdesc):
-        "Parse the HGH abinit header."
+        """Parse the HGH abinit header."""
         # Example:
         #Hartwigsen-Goedecker-Hutter psp for Ne,  from PRB58, 3641 (1998)
         #   10   8  010605 zatom,zion,pspdat
         # 3 1   1 0 2001 0  pspcod,pspxc,lmax,lloc,mmax,r2well
-
         lines = _read_nlines(filename, -1)
 
         header = _dict_from_lines(lines[:3], [0, 3, 6])
@@ -656,7 +646,7 @@ class NcAbinitHeader(AbinitHeader):
 
     @staticmethod
     def tm_header(filename, ppdesc):
-        "Parse the TM abinit header."
+        """Parse the TM abinit header."""
         # Example:
         #Troullier-Martins psp for element Fm         Thu Oct 27 17:28:39 EDT 1994
         #100.00000  14.00000    940714                zatom, zion, pspdat
@@ -670,17 +660,19 @@ class NcAbinitHeader(AbinitHeader):
         #   3  23.251  29.387    1   2.4776730        l,e99.0,e99.9,nproj,rcpsp
         #   .00000000    .0000000000    .0000000000    .00000000   rms,ekb1,ekb2,epsatm
         #   3.62474762267880     .07409391739104    3.07937699839200   rchrg,fchrg,qchrg
-
         lines = _read_nlines(filename, -1)
         header = []
 
         for (lineno, line) in enumerate(lines):
             header.append(line)
-            if lineno == 2: # Read lmax.
+            if lineno == 2: 
+                # Read lmax.
                 tokens = line.split()
-                pspcod, pspxc, lmax = map(int, tokens[:3])
-                if tokens[-1].strip() != "pspcod,pspxc,lmax,lloc,mmax,r2well":
-                    raise RuntimeError("%s: Invalid line\n %s"  % (filename, line))
+                pspcod, pspxc, lmax, lloc = map(int, tokens[:4])
+                mmax, r2well = map(float, tokens[4:6])
+                #if tokens[-1].strip() != "pspcod,pspxc,lmax,lloc,mmax,r2well":
+                #    raise RuntimeError("%s: Invalid line\n %s"  % (filename, line))
+
                 lines = lines[3:]
                 break
 
@@ -715,7 +707,7 @@ class PawAbinitHeader(AbinitHeader):
     """
     _attr_desc = collections.namedtuple("att", "default astype")
 
-    _vars = {
+    _VARS = {
         "zatom"           : _attr_desc(None, float),
         "zion"            : _attr_desc(None, float),
         "pspdat"          : _attr_desc(None, float),
@@ -738,12 +730,11 @@ class PawAbinitHeader(AbinitHeader):
     del _attr_desc
 
     def __init__(self, summary, **kwargs):
-
         super(PawAbinitHeader, self).__init__()
 
         self.summary = summary.strip()
 
-        for (key, desc) in PawAbinitHeader._vars.items():
+        for (key, desc) in PawAbinitHeader._VARS.items():
             default, astype = desc.default, desc.astype
 
             value = kwargs.pop(key, None)
@@ -765,7 +756,7 @@ class PawAbinitHeader(AbinitHeader):
 
     @staticmethod
     def paw_header(filename, ppdesc):
-        "Parse the PAW abinit header."
+        """Parse the PAW abinit header."""
         # Example
         #C  (US d-loc) - PAW data extracted from US-psp (D.Vanderbilt) - generated by USpp2Abinit v2.3.0
         #   6.000   4.000 20090106               : zatom,zion,pspdat
@@ -815,7 +806,7 @@ class PawAbinitHeader(AbinitHeader):
 ##########################################################################################
 
 class PseudoParserError(Exception):
-    pass
+    """Base Error class for the exceptions raised by `PseudoParser`"""
 
 class PseudoParser(object):
     """
@@ -826,11 +817,11 @@ class PseudoParser(object):
     """
     Error = PseudoParserError
 
-    #: Supported values of pspcod
+    # Supported values of pspcod
     ppdesc = collections.namedtuple("ppdesc", "pspcod name psp_type format")
 
     # TODO Recheck
-    _pspcodes = collections.OrderedDict( {
+    _PSPCODES = collections.OrderedDict( {
         1 : ppdesc(1, "TM",  "NC", None),
         3 : ppdesc(3, "HGH", "NC", None),
         #4 : ppdesc(4, "NC",     , None),
@@ -849,20 +840,21 @@ class PseudoParser(object):
         # List of files that could not been parsed.
         self._wrong_paths  = []
 
-    def scan_directory(self, dirname, exclude_exts=None):
+    def scan_directory(self, dirname, exclude_exts=(), exclude_fnames=()):
         """
         Analyze the files contained in directory dirname.
+
         Args:
             dirname:
                 directory path
             exclude_exts:
                 list of file extensions that should be skipped.
+            exclude_fnames:
+                list of file names that should be skipped.
 
-        :return: List of pseudopotential objects.
+        returns: 
+            List of pseudopotential objects.
         """
-        if exclude_exts is None:
-            exclude_exts = []
-
         for (i, ext) in enumerate(exclude_exts):
             if not ext.strip().startswith("."):
                 exclude_exts[i] =  "." + ext.strip()
@@ -871,8 +863,9 @@ class PseudoParser(object):
         paths = []
         for fname in os.listdir(dirname):
             root, ext = os.path.splitext(fname)
-            if ext not in exclude_exts:
-                paths.append(os.path.join(dirname, fname))
+            if ext in exclude_exts or fname in exclude_fnames or fname.startswith("."):
+                continue
+            paths.append(os.path.join(dirname, fname))
 
         pseudos = []
         for path in paths:
@@ -891,10 +884,12 @@ class PseudoParser(object):
         """
         Read the pseudopotential descriptor from file filename.
 
-        :return: Pseudopontential descriptor. None if filename is not a valid pseudopotential file.
-        :raise: `PseudoParserError` if fileformat is not supported.
-        """
+        Returns:
+            Pseudopontential descriptor. None if filename is not a valid pseudopotential file.
 
+        Raises:
+            `PseudoParserError` if fileformat is not supported.
+        """
         if filename.endswith(".xml"):
             raise self.Error("XML pseudo not supported yet")
 
@@ -917,10 +912,10 @@ class PseudoParser(object):
                         raise self.Error("%s: Invalid line\n %s"  % (filename, line))
                         return None
 
-                    if pspcod not in self._pspcodes:
+                    if pspcod not in self._PSPCODES:
                         raise self.Error("%s: Don't know how to handle pspcod %s\n"  % (filename, pspcod))
 
-                    ppdesc = PseudoParser._pspcodes[pspcod]
+                    ppdesc = self._PSPCODES[pspcod]
 
                     if pspcod == 7:
                         # PAW -> need to know the format pspfmt
@@ -939,29 +934,30 @@ class PseudoParser(object):
         """
         Read and parse a pseudopotential file.
 
-        :return: pseudopotential object or None if filename is not
-                 a valid pseudopotential file.
+        Returns: 
+            pseudopotential object or None if filename is not a valid pseudopotential file.
         """
         path = os.path.abspath(filename)
 
         ppdesc = self.read_ppdesc(path)
 
-        if ppdesc is None: return None
+        if ppdesc is None: 
+            return None
 
         psp_type = ppdesc.psp_type
 
         parsers = {
-         "FHI"              : NcAbinitHeader.fhi_header,
-         "TM"               : NcAbinitHeader.tm_header,
-         "HGH"              : NcAbinitHeader.hgh_header,
-         "HGHK"             : NcAbinitHeader.hgh_header,
-         "PAW_abinit_text"  : PawAbinitHeader.paw_header,
+         "FHI"            : NcAbinitHeader.fhi_header,
+         "TM"             : NcAbinitHeader.tm_header,
+         "HGH"            : NcAbinitHeader.hgh_header,
+         "HGHK"           : NcAbinitHeader.hgh_header,
+         "PAW_abinit_text": PawAbinitHeader.paw_header,
         }
 
-        #try:
-        header = parsers[ppdesc.name](path, ppdesc)
-        #except Exception as exc:
-        #    raise self.Error(str(exc))
+        try:
+            header = parsers[ppdesc.name](path, ppdesc)
+        except Exception as exc:
+            raise self.Error(filename + ": " + str(exc))
 
         root, ext = os.path.splitext(path)
 
@@ -1004,6 +1000,7 @@ class PseudoTable(collections.Sequence):
     """
     @classmethod
     def astable(cls, items):
+        """Return an instance of `PseudoTable from the iterable items.""" 
         if isinstance(items, cls):
             return items
         return cls(items)
@@ -1053,16 +1050,16 @@ class PseudoTable(collections.Sequence):
             return self._pseudos_with_z[Z]
 
     def __len__(self):
-        return len(list(self._iter__()))
+        return len(list(self.__iter__()))
 
     def __iter__(self):
-        "Process the elements in Z order."
+        """Process the elements in Z order."""
         for z in self.zlist:
             for pseudo in self._pseudos_with_z[z]:
                 yield pseudo
 
     def __repr__(self):
-        return "<%s at %s, long_name = %s>" % (self.__class__.__name__, id(self), self.long_name)
+        return "<%s at %s>" % (self.__class__.__name__, id(self))
 
     #def __str__(self):
     #    strio = StringIO.StringIO()
@@ -1072,28 +1069,24 @@ class PseudoTable(collections.Sequence):
 
     @property
     def allnc(self):
-        "True if all pseudos are norm-conserving"
+        """True if all pseudos are norm-conserving."""
         return all(p.isnc for p in self)
 
     @property
     def allpaw(self):
-        "True if all pseudos are PAW"
+        """True if all pseudos are PAW."""
         return all(p.ispaw for p in self)
 
-    @property
-    def long_name(self):
-        return "-".join([getattr(self, key) for key in ["psp_type", "xc_type", "name"]])
-
     @property
     def zlist(self):
-        "Ordered list with the atomic numbers available in the table."
+        """Ordered list with the atomic numbers available in the table."""
         zlist = list(self._pseudos_with_z.keys())
         zlist.sort()
         return zlist
 
     def iscomplete(self, zmax=118):
         """
-        True if table is complete i.e. all elements with z < zmax
+        True if table is complete i.e. all elements with Z < zmax
         have at least on pseudopotential
         """
         for z in range(1, zmax):
@@ -1111,7 +1104,7 @@ class PseudoTable(collections.Sequence):
             return []
 
     def pseudo_from_name(self, name):
-        "Return the pseudo in the table with the given name"
+        """Return the pseudo in the table with the given name"""
         for pseudo in self:
             if pseudo.name == name:
                 return pseudo
@@ -1128,12 +1121,10 @@ class PseudoTable(collections.Sequence):
                 Template for displaying the element properties, with one
                 % for each property.
 
-        :Returns: None
-
         For example, print a table of mass and density.
 
         from periodictable import elements
-        elements.list('symbol','mass','density', format="%-2s: %6.2f u %5.2f g/cm^3")
+        elements.list_properties('symbol','mass','density', format="%-2s: %6.2f u %5.2f g/cm^3")
         H :   1.01 u   0.07 g/cm^3
         He:   4.00 u   0.12 g/cm^3
         Li:   6.94 u   0.53 g/cm^3
@@ -1145,23 +1136,23 @@ class PseudoTable(collections.Sequence):
 
         for pseudo in self:
             try:
-                L = tuple(getattr(pseudo,p) for p in props)
+                values = tuple(getattr(pseudo, p) for p in props)
             except AttributeError:
                 # Skip elements which don't define all the attributes
                 continue
 
             # Skip elements with a value of None
-            if any(v is None for v in L):
+            if any(v is None for v in values):
                 continue
 
             if format is None:
-                print(" ".join(str(p) for p in L))
+                print(" ".join(str(p) for p in values))
             else:
-                #try:
-                print(format % L)
-                #except:
-                #    print "format",format,"args",L
-                #    raise
+                try:
+                    print(format % values)
+                except:
+                    print("format",format,"args",values)
+                    raise
 
     def print_table(self, stream=sys.stdout, filter_function=None):
         """
@@ -1169,9 +1160,9 @@ class PseudoTable(collections.Sequence):
 
         Args:
             filter_function:
-                A filtering function taking a Pseudo as input and returns a boolean.
-                For example, setting filter_function = lambda el: el.X > 2 will print
-                a periodic table containing only elements with electronegativity > 2.
+                A filtering function that take a Pseudo as input and returns a boolean.
+                For example, setting filter_function = lambda el: el.Z_val > 2 will print
+                a periodic table containing only pseudos with Z_val > 2.
         """
         for row in range(1, 10):
             rowstr = []
@@ -1183,193 +1174,31 @@ class PseudoTable(collections.Sequence):
                     rowstr.append("   ")
             print(" ".join(rowstr))
 
-##########################################################################################
+    def sorted(self, attrname, reverse=False):
+        """Sort the table according to the value of attribute attrname."""
+        attrs = []
+        for i, pseudo in self:
+            try:
+                a = getattr(pseudo, attrname)
+            except AttributeError:
+                a = np.inf
+            attrs.append((i, a))
 
-class PseudoDatabase(dict):
+        # Sort attrs, and build new table with sorted pseudos.
+        attrs = sorted(attrs, key=lambda t:t[1], reverse=reverse)
+        return PseudoTable([self[a[0]] for a in attrs])
 
-    _save_file = "pseudo_database.pickle"
-    #
-    #        "TM"
-    #"LDA"   "HGH"
-    #"GGA"   "HGK"
-    #        "FHI"
-    #        "USERS"
+    def select(self, condition):
+        """
+        Select only those pseudopotentials for which condition is True.
 
-    #"LDA"   "ATOMPAW"
-    #"GGA"   "USPP"
-    #        "USERS"
+        Args:
+            condition:
+                Function that accepts a `Pseudo` object and returns True or False.
+        """
+        return PseudoTable([p for p in self if condition(p)])
 
-    # xc_type = xc_family-[xc_flavor]
-    # dirname = psp_type _ xc_type _ table_name
-    PSP_TYPES = ["NC", "PAW"]
+    def with_dojo_report(self):
+        """Select pseudos containing the DOJO_REPORT section."""
+        return self.select(condition=lambda p : p.has_dojo_report)
 
-    XC_FAMILIES = ["LDA", "GGA"]
-
-    XC_FLAVOR   = ["PBE", "PW91"]
-
-    XC_TYPES = ["LDA", "GGA"]
-
-    def __new__(cls, dirpath=None, force_reload=False):
-        new = dict.__new__(cls)
-
-        if dirpath is None:
-            return new
-
-        dirpath = os.path.abspath(dirpath)
-
-        cached_database = os.path.join(dirpath, cls._save_file)
-
-        if not os.path.exists(cached_database) or force_reload:
-            new = PseudoDatabase.__build(new, dirpath)
-        else:
-            new = PseudoDatabase.__from_filename(new, cached_database)
-
-        return new
-
-    def __init__(self, dirpath=None, force_reload=False):
-        pass
-
-    def __len__(self):
-        return len([self.all_pseudos])
-
-    def __build(self, top):
-        print("Building new database...")
-
-        new_database = PseudoDatabase()
-
-        for key in self.PSP_TYPES:
-            new_database[key] = None
-
-        parser = PseudoParser()
-
-        for psp_type in self.PSP_TYPES:
-            new_database[psp_type] = dict.fromkeys(self.XC_TYPES, dict())
-
-        for (dirpath, dirnames, filenames) in os.walk(top):
-            for dirname in dirnames:
-
-                try:
-                    psp_type, xc_type, table_type = os.path.basename(dirname).split("_")
-                except:
-                    err_msg = "Malformatted name for directory %s" % dirname
-                    raise RuntimeError(err_msg)
-                #print(psp_type, xc_type, table_type)
-
-                if psp_type not in self.PSP_TYPES or xc_type not in self.XC_TYPES:
-                    raise ValueError("Don't know how to handle %s %s" % (psp_type, xc_type))
-
-                pseudos = parser.scan_directory( os.path.join(dirpath, dirname),
-                    exclude_exts = [".py", ".ini", ".sh", ".gz", ".pl", ".txt", ".swp", ".data", "pickle",])
-
-                table = PseudoTable(pseudos)
-
-                new_database[psp_type][xc_type][table_type] = table
-
-        # Save the database.
-        cached_database = os.path.join(top, self._save_file)
-
-        new_database.save(cached_database)
-
-        return new_database
-
-    def __from_filename(self, filename):
-
-        cached_database = filename
-        if cached_database is None: cached_database = self._save_file
-
-        print("Loading database from: %s" % cached_database)
-
-        # Read the database from the cpickle file.
-        # Use file locking mechanism to prevent IO from other processes.
-        #with FileLock(database_path + ".lock") as lock:
-        with open(cached_database, "r") as fh:
-            database = pickle.load(fh)
-
-        #assert database.dirpath == os.path.split(abspath(cached_database))[0]
-        return database
-
-    @property
-    def LDA_HGH_PPTABLE(self):
-        return self["NC"]["LDA"]["HGH"]
-
-    @property
-    def GGA_FHI_PPTABLE(self):
-        return self["NC"]["GGA"]["FHI"]
-
-    @property
-    def GGA_HGHK_PPTABLE(self):
-        return self["NC"]["GGA"]["HGHK"]
-
-    @property
-    def path(self):
-        return self._path
-
-    @property
-    def dirname(self):
-        return os.path.dirname(self._path)
-
-    def save(self, filename=None, protocol=-1):
-
-        cached_database = filename
-        if cached_database is None: cached_database = self._save_file
-
-        print("Saving database on file %s" % cached_database)
-
-        # Save the database in the cpickle file.
-        # Use file locking mechanism to prevent IO from other processes.
-        #with FileLock(cached_database) as lock:
-
-        self._path = cached_database
-
-        with open(cached_database, "w") as fh:
-            pickle.dump(self, fh, protocol=protocol)
-
-    @property
-    def all_pseudos(self):
-        "Return a list with all the pseudopotentials in the database"
-        pseudos = []
-        for (k, table) in nested_dict_items(self):
-            pseudos.extend([p for p in table])
-        return pseudos
-
-    def write_hash_table(self, filename):
-        #with open(filename, "w") as fh
-        fh = sys.stdout
-
-        def tail2(path):
-            head, tail0 = os.path.split(path)
-            head, tail1 = os.path.split(head)
-            return os.path.join(tail1, tail0)
-
-        fh.write("# relative_path, md5 num_line\n")
-        for pseudo in self.all_pseudos():
-            #print type(pseudo), pseudo
-            checksum = pseudo.checksum()
-            relative_path = tail2(pseudo.path)
-            fh.write("%s %s %s\n" % (relative_path, checksum[0], checksum[1]))
-
-    def table(self, psp_type, xc_type):
-        return self[psp_type][xc_type].values()
-
-    def nc_tables(self, xc_type):
-        "Iterate over the norm-conserving tables with XC type xc_type."
-        return self.table("NC", xc_type)
-
-    def paw_tables(self, psp_type, xc_type):
-        "Iterate over the PAW tables with XC type xc_type."
-        return self.table("PAW", xc_type)
-
-    def nc_pseudos(self, symbol, xc_type, table_type=None, **kwargs):
-        "Return a list of :class:`Pseudo` instances."
-        pseudos = []
-        for table in self.nc_tables(xc_type):
-            if table_type is not None and table_type != table.type: continue
-            pseudos.extend( table.pseudos_with_symbol(symbol) )
-
-        return pseudos
-
-    #def paw_pseudos(self, symbol, xc_type, table_type=None, **kwargs):
-
-    #def find_all(self, symbol, xc_type):
-
-##########################################################################################
