@@ -32,7 +32,7 @@ class ExecWrapper(object):
             executable:
                 path to the executable.
             verbose:
-                Verbosity level
+                Verbosity level.
         """
         if executable is None: executable = self.name
 
@@ -76,13 +76,12 @@ class ExecWrapper(object):
 
 ##########################################################################################
 
+class MrgscrError(Exception):
+    """Error class for Mrgscr"""
 
 class Mrgscr(ExecWrapper):
     _name = "mrgscr"
 
-    class MrgscrError(Exception):
-        """Error class for Mrgscr"""
-
     Error = MrgscrError
 
     def merge_qpoints(self, files_to_merge, out_prefix, cwd=None):
@@ -132,13 +131,13 @@ class Mrgscr(ExecWrapper):
 
 ##########################################################################################
 
+class MrggkkError(Exception):
+    """Error class for Mrggkk."""
+
 
 class Mrggkk(ExecWrapper):
     _name = "mrggkk"
 
-    class MrggkkError(Exception):
-        """Error class for Mrggkk."""
-
     Error = MrggkkError
 
     def merge(self, gswfk_file, dfpt_files, gkk_files, out_fname, binascii=0, cwd=None):
@@ -195,13 +194,12 @@ class Mrggkk(ExecWrapper):
 
 ##########################################################################################
 
+class MrgddbError(Exception):
+    """Error class for Mrgddb."""
 
 class Mrgddb(ExecWrapper):
     _name = "mrgddb"
 
-    class MrgddbError(Exception):
-        """Error class for Mrgddb."""
-
     Error = MrgddbError
 
     def merge(self, ddb_files, out_ddb, description, cwd=None):
@@ -256,13 +254,12 @@ class Mrgddb(ExecWrapper):
 
 ##########################################################################################
 
+class AnaddbError(Exception):
+    """Error class for Anaddb."""
 
 class Anaddb(ExecWrapper):
     _name = "anaddb"
 
-    class AnaddbError(Exception):
-        """Error class for Anaddb."""
-
     Error = AnaddbError
 
     #def make_stdin(self):
