@@ -30,6 +30,7 @@ from pymatgen.serializers.json_coders import MSONable
 from pymatgen.symmetry.finder import SymmetryFinder
 from pymatgen.symmetry.bandstructure import HighSymmKpath
 from pymatgen.util.decorators import deprecated
+from pymatgen import write_structure
 import traceback
 import numpy as np
 import shutil
@@ -476,6 +477,85 @@ class MITHSEVaspInputSet(VaspInputSet):
         return cls(user_incar_settings=d["user_incar_settings"],
                    constrain_total_magmom=d["constrain_total_magmom"])
 
+class MITNEBVaspInputSet(VaspInputSet):
+    """
+    Class for writing NEB inputs.
+    """
+    def __init__(self, user_incar_settings=None, constrain_total_magmom=False,
+                 ggau=True, nimages=8):
+        module_dir = os.path.dirname(os.path.abspath(__file__))
+        with open(os.path.join(module_dir, "MITVaspInputSet.json")) as f:
+            DictVaspInputSet.__init__(
+                self, "MIT NEB", json.load(f),
+                constrain_total_magmom=constrain_total_magmom)
+        self.user_incar_settings = user_incar_settings
+        #incar settings
+        incar_settings = {'IMAGES':nimages, 'IBRION':1, 'NFREE':2}
+        self.incar_settings.update(incar_settings)
+        if user_incar_settings:
+            self.incar_settings.update(user_incar_settings)
+        if not ggau:
+            self.incar_settings['LDAU'] = False
+            if 'LDAUU' in self.incar_settings:
+                # technically not needed, but clarifies INCAR
+                del self.incar_settings['LDAUU']
+                del self.incar_settings['LDAUJ']
+                del self.incar_settings['LDAUL']
+                
+    def write_input(self, structures, output_dir, make_dir_if_not_present=True,
+                    write_cif=False):
+        """
+        Writes a set of VASP input to a directory.
+
+        Args:
+            structures:
+                list of Structure objects. There should be nimages+2 structures
+            output_dir:
+                Directory to output the VASP input files
+            make_dir_if_not_present:
+                Set to True if you want the directory (and the whole path) to
+                be created if it is not present.
+            write_cif:
+                If true, writes a cif along with each POSCAR
+        """
+        if len(structures) != self.incar_settings['IMAGES'] + 2:
+            raise ValueError('incorrect number of structures')
+        if make_dir_if_not_present and not os.path.exists(output_dir):
+            os.makedirs(output_dir)
+        self.get_incar(structures[0]).write_file(os.path.join(output_dir, 
+                                                              'INCAR'))
+        self.get_kpoints(structures[0]).write_file(os.path.join(output_dir, 
+                                                                'KPOINTS'))
+        self.get_potcar(structures[0]).write_file(os.path.join(output_dir, 
+                                                               'POTCAR'))
+        for i, s in enumerate(structures):
+            d = os.path.join(output_dir, str(i).zfill(2))
+            if make_dir_if_not_present and not os.path.exists(d):
+                os.makedirs(d)
+            self.get_poscar(s).write_file(os.path.join(d, 'POSCAR'))
+            if write_cif:
+                write_structure(s, os.path.join(d, '{}.cif'.format(i)))
+                
+    @property
+    def to_dict(self):
+        return {
+            "name": self.name,
+            "constrain_total_magmom": self.set_nupdown,
+            "user_incar_settings": self.user_incar_settings,
+            "ggau": self.ggau,
+            "nimages": self.nimages,
+            "@class": self.__class__.__name__,
+            "@module": self.__class__.__module__,
+        }
+
+    @classmethod
+    def from_dict(cls, d):
+        return cls(user_incar_settings=d["user_incar_settings"],
+                   constrain_total_magmom=d["constrain_total_magmom"],
+                   ggau=d["ggau"],
+                   nimages=d["nimages"])
+    
+
 
 class MITMDVaspInputSet(VaspInputSet):
     """
@@ -658,11 +738,12 @@ class MPStaticVaspInputSet(MPVaspInputSet):
                 constrain_total_magmom=constrain_total_magmom)
 
         self.user_incar_settings = user_incar_settings
-        self.incar_settings.update(
-            {"IBRION": -1, "ISMEAR": -5, "LAECHG": True, "LCHARG": True,
-             "LORBIT": 11, "LVHAR": True, "LWAVE": False, "NSW": 0})
         if user_incar_settings:
             self.incar_settings.update(user_incar_settings)
+        self.incar_settings.update(
+            {"IBRION": -1, "ISMEAR": -5, "LAECHG": True, "LCHARG": True,
+             "LORBIT": 11, "LVHAR": True, "LWAVE": False, "NSW": 0, "ICHARG":0,
+             "EDIFF": 0.000001})
 
     def get_kpoints(self, structure, kpoints_density=90):
         """
@@ -735,7 +816,7 @@ class MPStaticVaspInputSet(MPVaspInputSet):
 
     @staticmethod
     def from_previous_vasp_run(previous_vasp_dir, output_dir='.',
-                               user_incar_settings=None,
+                               user_incar_settings={},
                                make_dir_if_not_present=True):
         """
         Generate a set of Vasp input files for static calculations from a
@@ -757,16 +838,36 @@ class MPStaticVaspInputSet(MPVaspInputSet):
             vasp_run = Vasprun(os.path.join(previous_vasp_dir, "vasprun.xml"),
                                parse_dos=False, parse_eigen=None)
             outcar = Outcar(os.path.join(previous_vasp_dir, "OUTCAR"))
+            previous_incar = Incar.from_file(os.path.join(previous_vasp_dir, "INCAR"))
+            kpoints = Kpoints.from_file(os.path.join(previous_vasp_dir, "KPOINTS"))
         except:
             traceback.format_exc()
             raise RuntimeError("Can't get valid results from previous run")
 
         structure = MPStaticVaspInputSet.get_structure(
             vasp_run, outcar)
-        mpsvip = MPStaticVaspInputSet(
-            user_incar_settings=user_incar_settings)
+        mpsvip = MPStaticVaspInputSet()
         mpsvip.write_input(structure, output_dir, make_dir_if_not_present)
+        new_incar = Incar.from_file(os.path.join(output_dir, "INCAR"))
 
+        # Use previous run INCAR and override necessary parameters
+        previous_incar.update({"IBRION": -1, "ISMEAR": -5, "LAECHG": True, "LCHARG": True,
+             "LORBIT": 11, "LVHAR": True, "LWAVE": False, "NSW": 0, "ICHARG":0})
+
+        # Compare ediff between previous and staticinputset values, choose the tigher ediff
+        previous_incar.update({"EDIFF": min(previous_incar["EDIFF"], new_incar["EDIFF"])})
+
+        # add user settings
+        if user_incar_settings:
+            previous_incar.update(user_incar_settings)
+        previous_incar.write_file(os.path.join(output_dir, "INCAR"))
+
+        # Prefer to use k-point scheme from previous run
+        kpoints_out = Kpoints.from_file(os.path.join(output_dir, "KPOINTS"))
+        if kpoints.style != kpoints_out.style:
+            if SymmetryFinder(structure, symprec=0.01).get_lattice_type() != "hexagonal":
+                k_div = (kp+1 if kp%2==1 else kp for kp in kpoints_out.kpts[0])
+                Kpoints.monkhorst_automatic(k_div).write_file(os.path.join(output_dir, "KPOINTS"))
 
 class MPNonSCFVaspInputSet(MPStaticVaspInputSet):
     """
