@@ -65,7 +65,6 @@ class SymmetryFinder(object):
         self._symprec = symprec
         self._angle_tol = angle_tolerance
         self._structure = structure
-
         #Spglib"s convention for the lattice definition is the transpose of the
         #pymatgen version.
         self._transposed_latt = structure.lattice.matrix.transpose()
@@ -274,7 +273,6 @@ class SymmetryFinder(object):
             symmops.append(op)
         return symmops
 
-
     def get_point_group_operations(self, cartesian=False):
         """
         Return symmetry operations as a list of SymmOp objects.
@@ -288,10 +286,10 @@ class SymmetryFinder(object):
         for rot, trans in zip(rotation, translation):
             if cartesian:
                 rot = np.dot(mat, np.dot(rot, invmat))
-                trans = np.dot(trans, self._structure.lattice.matrix)
-            op = SymmOp.from_rotation_and_translation(rot, np.array([0,0,0]))
+            op = SymmOp.from_rotation_and_translation(rot, np.array([0, 0, 0]))
             symmops.append(op)
         return symmops
+
     def get_symmetrized_structure(self):
         """
         Get a symmetrized structure. A symmetrized structure is one where the
@@ -352,8 +350,8 @@ class SymmetryFinder(object):
         species = [self._unique_species[i - 1] for i in zs]
 
         if num_atom_prim > 0:
-            return Structure(lattice.T, species, positions[:num_atom_prim])\
-                .get_reduced_structure()
+            return Structure(lattice.T, species, positions[:num_atom_prim],
+                             to_unit_cell=True).get_reduced_structure()
         else:
             #Not sure if we should return None or just return the full
             #structure.
@@ -763,7 +761,7 @@ class SymmetryFinder(object):
 
         new_coords = np.dot(transf, np.transpose(struct.frac_coords)).T
         new_struct = Structure(latt, struct.species_and_occu, new_coords,
-                               site_properties=struct.site_properties)
+                               site_properties=struct.site_properties,to_unit_cell=True)
         return new_struct.get_sorted_structure()
 
 
